@@ -40,6 +40,7 @@
 
 #include <stdarg.h>
 #include <stdint.h>
+#include <errno.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -75,6 +76,11 @@ extern int nbdkit_parse_bool (const char *str);
 extern int nbdkit_read_password (const char *value, char **password);
 extern char *nbdkit_realpath (const char *path);
 
+/* A static non-NULL pointer which can be used when you don't need a
+ * per-connection handle.
+ */
+#define NBDKIT_HANDLE_NOT_NEEDED (&errno)
+
 #ifdef __cplusplus
 }
 #endif
