@@ -105,7 +105,8 @@ static int
 example2_config_complete (void)
 {
   if (filename == NULL) {
-    nbdkit_error ("you must supply the file=<FILENAME> parameter after the plugin name on the command line");
+    nbdkit_error ("you must supply the file=<FILENAME> parameter "
+                  "after the plugin name on the command line");
     return -1;
   }
 
