@@ -102,6 +102,17 @@ function_defined (const char *name)
 static void
 lua_plugin_dump_plugin (void)
 {
+#ifdef LUA_VERSION_MAJOR
+  printf ("lua_version=%s", LUA_VERSION_MAJOR);
+#ifdef LUA_VERSION_MINOR
+  printf (".%s", LUA_VERSION_MINOR);
+#ifdef LUA_VERSION_RELEASE
+  printf (".%s", LUA_VERSION_RELEASE);
+#endif
+#endif
+  printf ("\n");
+#endif
+
   if (script && function_defined ("dump_plugin")) {
     lua_getglobal (L, "dump_plugin");
     if (lua_pcall (L, 0, 0, 0) != 0) {
@@ -143,7 +154,10 @@ lua_plugin_config (const char *key, const char *value)
     if (!function_defined ("open") ||
         !function_defined ("get_size") ||
         !function_defined ("pread")) {
-      nbdkit_error ("%s: one of the required callbacks 'open', 'get_size' or 'pread' is not defined by this Lua script.  nbdkit requires these callbacks.", script);
+      nbdkit_error ("%s: one of the required callbacks "
+                    "'open', 'get_size' or 'pread' "
+                    "is not defined by this Lua script.  "
+                    "nbdkit requires these callbacks.", script);
       return -1;
     }
   }
