@@ -49,18 +49,8 @@ main (int argc, char *argv[])
 {
   guestfs_h *g;
   int r;
-  const char *s;
   char *data;
 
-  /* The ext2 test fails valgrind.  It seems as if the ext2fs error
-   * table cannot be freed.
-   */
-  s = getenv ("NBDKIT_VALGRIND");
-  if (s && strcmp (s, "1") == 0) {
-    fprintf (stderr, "ext2 test skipped under valgrind.\n");
-    exit (77);                  /* Tells automake to skip the test. */
-  }
-
   if (test_start_nbdkit ("ext2", "-r", "disk=ext2.img", "file=/disks/disk.img",
                          NULL) == -1)
     exit (EXIT_FAILURE);
