@@ -55,7 +55,7 @@ main (int argc, char *argv[])
   int r;
   int fd;
   char cmd[64], buf[64];
-  char disk[] = "/tmp/diskXXXXXX"; /* Backing disk. */
+  char disk[] = LARGE_TMPDIR "/diskXXXXXX"; /* Backing disk. */
   FILE *pp;
   char *data;
   size_t len;
@@ -124,9 +124,8 @@ main (int argc, char *argv[])
   atexit (detach_loopdev);
 
   /* Start nbdkit. */
-  snprintf (buf, sizeof buf, "file=%s", loopdev);
   if (test_start_nbdkit ("-D", "file.zero=1",
-                         "file", buf, NULL) == -1)
+                         "file", loopdev, NULL) == -1)
     exit (EXIT_FAILURE);
 
   g = guestfs_create ();
@@ -166,7 +165,9 @@ main (int argc, char *argv[])
     exit (EXIT_FAILURE);
 
   if (strcmp (data, content) != 0) {
-    fprintf (stderr, "%s FAILED: unexpected content of %s file (actual: %s, expected: %s)\n",
+    fprintf (stderr,
+             "%s FAILED: unexpected content of %s file "
+             "(actual: %s, expected: %s)\n",
              program_name, filename, data, content);
     exit (EXIT_FAILURE);
   }
