@@ -34,10 +34,12 @@
 # Test the memory plugin with the largest possible size supported
 # by nbdkit.
 
+source ./functions.sh
 set -e
 
 files="memory-largest.out memory-largest.pid memory-largest.sock"
 rm -f $files
+cleanup_fn rm -f $files
 
 # Test that qemu-io works
 if ! qemu-io --help >/dev/null; then
@@ -47,38 +49,8 @@ fi
 
 # Run nbdkit with memory plugin.
 # size = 2^63-1
-nbdkit -f -v \
-       -D memory.dir=1 \
-       -P memory-largest.pid -U memory-largest.sock \
-       memory size=9223372036854775807 &
-
-# We may have to wait a short time for the pid file to appear.
-for i in `seq 1 10`; do
-    if test -f memory-largest.pid; then
-        break
-    fi
-    sleep 1
-done
-if ! test -f memory-largest.pid; then
-    echo "$0: PID file was not created"
-    exit 1
-fi
-
-pid="$(cat memory-largest.pid)"
-
-# Kill the nbdkit process on exit.
-cleanup ()
-{
-    status=$?
-    trap '' INT QUIT TERM EXIT ERR
-    echo $0: cleanup: exit code $status
-
-    kill $pid
-    rm -f $files
-
-    exit $status
-}
-trap cleanup INT QUIT TERM EXIT ERR
+start_nbdkit -P memory-largest.pid -U memory-largest.sock \
+       memory size=9223372036854775807
 
 # qemu cannot open this image!
 #
@@ -87,5 +59,3 @@ trap cleanup INT QUIT TERM EXIT ERR
 # Therefore we skip the remainder of this test (in effect, testing
 # only that nbdkit can create the file).
 exit 77
-
-# The cleanup() function is called implicitly on exit.
