@@ -52,7 +52,7 @@ main (int argc, char *argv[])
   char *data;
   size_t i, size;
 
-  if (test_start_nbdkit ("-o", "file", "file=file-data", NULL) == -1)
+  if (test_start_nbdkit ("-o", "file", "file-data", NULL) == -1)
     exit (EXIT_FAILURE);
 
   g = guestfs_create ();
@@ -78,7 +78,8 @@ main (int argc, char *argv[])
   if (!data)
     exit (EXIT_FAILURE);
   if (size != 8 * 512) {
-    fprintf (stderr, "%s FAILED: unexpected size (actual: %zu, expected: 512)\n",
+    fprintf (stderr,
+             "%s FAILED: unexpected size (actual: %zu, expected: 512)\n",
              program_name, size);
     exit (EXIT_FAILURE);
   }
