@@ -34,10 +34,12 @@
 # Test the pattern plugin with the largest possible size supported
 # by nbdkit.
 
+source ./functions.sh
 set -e
 
 files="pattern-largest.out pattern-largest.pid pattern-largest.sock"
 rm -f $files
+cleanup_fn rm -f $files
 
 # Test that qemu-io works
 if ! qemu-io --help >/dev/null; then
@@ -47,37 +49,9 @@ fi
 
 # Run nbdkit with pattern plugin.
 # size = 2^63-1
-nbdkit -P pattern-largest.pid -U pattern-largest.sock \
+start_nbdkit -P pattern-largest.pid -U pattern-largest.sock \
        pattern size=9223372036854775807
 
-# We may have to wait a short time for the pid file to appear.
-for i in `seq 1 10`; do
-    if test -f pattern-largest.pid; then
-        break
-    fi
-    sleep 1
-done
-if ! test -f pattern-largest.pid; then
-    echo "$0: PID file was not created"
-    exit 1
-fi
-
-pid="$(cat pattern-largest.pid)"
-
-# Kill the nbdkit process on exit.
-cleanup ()
-{
-    status=$?
-    trap '' INT QUIT TERM EXIT ERR
-    echo $0: cleanup: exit code $status
-
-    kill $pid
-    rm -f $files
-
-    exit $status
-}
-trap cleanup INT QUIT TERM EXIT ERR
-
 # qemu cannot open this image!
 #
 #   can't open device nbd+unix://?socket=pattern-largest.sock: Could not get image size: File too large
@@ -97,5 +71,3 @@ then
     cat pattern-largest.out
     exit 1
 fi
-
-# The cleanup() function is called implicitly on exit.
