@@ -31,9 +31,9 @@
 # OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 # SUCH DAMAGE.
 
+source ./functions.sh
 set -e
 set -x
-source ./functions.sh
 
 if ! socat -h; then
     echo "$0: 'socat' command not available"
@@ -51,24 +51,18 @@ rm -f $files
 truncate -s 1G single.disk
 
 socat unix-listen:single.sock,reuseaddr,fork \
-    exec:'nbdkit -r -s file file=single.disk' &
+    exec:'nbdkit -r -s file single.disk' &
 pid=$!
 
 cleanup ()
 {
-    status=$?
-    trap '' INT QUIT TERM EXIT ERR
-    echo $0: cleanup: exit code $status
-
     kill $pid
     rm -f $files
-
-    exit $status
 }
-trap cleanup INT QUIT TERM EXIT ERR
+cleanup_fn cleanup
 
 # Wait for socat to start up and create the socket.
-for i in `seq 1 10`; do
+for i in {1..10}; do
     if test -S single.sock; then
         break
     fi
