@@ -33,6 +33,7 @@
 
 source ./functions.sh
 set -e
+set -x
 
 output="$(nbdkit --version)"
 if [[ ! ( "$output" =~ ^nbdkit\ 1\. ) ]]; then
@@ -48,13 +49,10 @@ test ()
     vg=; [ "$NBDKIT_VALGRIND" = "1" ] && vg="-valgrind"
     case "$1$vg" in
         vddk | vddk-valgrind)
-            # VDDK won't run without special environment variables
-            # being set, so ignore it.
+            echo "$0: skipping $1$vg because VDDK cannot run without special environment variables"
             ;;
-        lua-valgrind | perl-valgrind | python-valgrind | \
-        ruby-valgrind | tcl-valgrind | \
-        example4-valgrind | tar-valgrind)
-            # Plugins written in scripting languages can't run under valgrind.
+        python-valgrind | ruby-valgrind | tcl-valgrind)
+            echo "$0: skipping $1$vg because this language doesn't support valgrind"
             ;;
         *)
             nbdkit $1 --version
