@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013 Red Hat Inc.
+ * Copyright (C) 2013-2018 Red Hat Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -51,7 +51,7 @@ main (int argc, char *argv[])
   int r;
   char *data;
 
-  if (test_start_nbdkit ("xz", "-r", "disk.xz", NULL) == -1)
+  if (test_start_nbdkit ("--filter=xz", "file", "disk.xz", NULL) == -1)
     exit (EXIT_FAILURE);
 
   g = guestfs_create ();
