@@ -39,13 +39,14 @@
 
 #define NBDKIT_START_TIMEOUT 30 /* seconds */
 
-#define NBDKIT_PLUGIN(name) \
-  "../plugins/" name "/.libs/nbdkit-" name "-plugin.so"
-
-extern pid_t pid;               /* PID of nbdkit process. */
+extern pid_t pid;               /* PID of most recent nbdkit process */
+extern const char *sock;        /* socket of most recent nbdkit process */
 extern const char *server[2];   /* server parameter for add_drive */
 
-extern int test_start_nbdkit (const char *arg, ...);
+/* Can be called more than once (useful for nbd plugin) */
+extern int test_start_nbdkit (const char *arg, ...)
+  __attribute__((__nonnull__ (1)))
+  __attribute__((__sentinel__));
 
 /* Declare program_name. */
 #if HAVE_DECL_PROGRAM_INVOCATION_SHORT_NAME == 1
