@@ -1,11 +1,12 @@
 type handle = {
-  disk : string;                        (* The disk image. *)
+  disk : bytes;                 (* The disk image. *)
 }
 
 let test_load () =
-  ()
+  NBDKit.debug "test ocaml plugin loaded"
+
 let test_unload () =
-  ()
+  NBDKit.debug "test ocaml plugin unloaded"
 
 let params = ref []
 
@@ -17,27 +18,21 @@ let test_config_complete () =
   assert (params = [ "a", "1"; "b", "2"; "c", "3" ])
 
 let test_open readonly =
-  let disk =
-    let s = String.create (512 * 8) in
-    for i = 0 to 511 do
-      for j = 0 to 7 do
-        s.[i*8+j] <- Char.chr (j+1)
-      done
-    done;
-    s in
-  { disk = disk }
+  NBDKit.debug "test ocaml plugin handle opened readonly=%b" readonly;
+  let disk = Bytes.make (1024*1024) '\000' in
+  { disk }
 
 let test_close h =
   ()
 
 let test_get_size h =
-  Int64.of_int (String.length h.disk)
+  Int64.of_int (Bytes.length h.disk)
 
-let test_pread h buf offset =
-  let len = String.length buf in
-  String.blit h.disk (Int64.to_int offset) buf 0 len
+let test_pread h buf offset _ =
+  let len = Bytes.length buf in
+  Bytes.blit h.disk (Int64.to_int offset) buf 0 len
 
-let test_pwrite h buf offset =
+let test_pwrite h buf offset _ =
   let len = String.length buf in
   String.blit buf 0 h.disk (Int64.to_int offset) len
 
